/*
<trigger>
	<usage>bsf ecp_pre_duplicate.js</usage>
	<version>5.0</version>
	<description>
		現在の CR を元の CR の重複にすることができる場合に、属性 _DUPLICATE に格納される数値を決定します。
		現在の CR に重複 CR があってはなりません。また、元の CR は、現在のデータベース内で変更可能でなければならず、「重複」という文字列で始まる状態であってはなりません。
	</description>
</trigger>
*/

// Import packages
importPackage(Packages.com.telelogic.cs.api);

// Script specific global variables
const script_name = 'ecp_pre_duplicate.js';
const script_version = '20080604';
const script_usage = 'ecp_pre_duplicate.js';
const script_example = 'ecp_pre_duplicate.js';
const log_header = 'Script bsf ' + script_name + ': ';
const script_debug = false;
const script_dcm_modifiable_check = true;

// Get objects
var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');
var options = OptionParser.parse(args);

if (script_debug) 
{
	log.info(log_header);
	log.info('    Version: ' + script_version);
	log.info('    Arguments: ' + args.join(' '));
	log.info('    Current CR: ' + event.objectId);
}


if (checkArguments(0, [], []))
{
	// Test current CR for having duplicate CRs
	const relationship = 'duplicate';
	var duplicates = change.getRelatedFrom(event.triggerUser.token, event.objectId, relationship, []);
	
	if (script_debug) 
	{
		log.info(log_header + 'Found ' + duplicates.length + ' duplicate CRs to the current CR');
	}
	
	if (duplicates.length > 0)
	{
		event.stopTransition(change.getMessage('errorDuplicatingHasDuplicates'));
	}
	else
	{
		// Find original CR - get id
		const original_attr_name = '_DUPLICATE';
		var original_attr = change.collectAttributes(event.triggerUser.token, event, [original_attr_name]).get(original_attr_name);
	
		if (script_debug)
		{
			log.info(log_header + 'The original CR attribute has the value \'' + original_attr + '\'');
		}
		
		if (isAttributeNull(original_attr))
		{
			event.stopTransition(change.getMessage('errorDuplicatingNoOriginalCR'));
		}
		else
		{
			// Find original CR - get CR
			const modifiable_attr_name = 'modifiable_in';
			const crstatus_attr_name = 'crstatus';
			var attributes = [crstatus_attr_name];
		
			if (script_dcm_modifiable_check)
			{
				attributes.push(modifiable_attr_name);
			}

			var originals = change.query(event.triggerUser.token, change.buildCrQuery(event.triggerUser.token, original_attr), attributes);
			
			if (script_debug)
			{
				log.info(log_header + 'Found ' + originals.length + ' CRs querying for the original CR');
			}
			
			if (originals.length != 1)
			{
				event.stopTransition(change.getMessage('errorDuplicatingOriginalCRXnotFound', [original_attr]));
			}
			else
			{
				// Test for being modifiable
				// modifibale_in is never null, if DCM is enabled
				if (script_dcm_modifiable_check &&
					change.isDcmEnabled(event.triggerUser.token) &&
					!originals[0].getAttribute(modifiable_attr_name).toString().equals(change.getDbid(event.triggerUser.token))) // == is always false
				{
					if (script_debug)
					{
						log.info(log_header + 'The original CR is not modifiable in the database');
					}

					event.stopTransition(change.getMessage('errorDuplicatingOriginalCRXnotModifiable', [original_attr]));
				}
				else
				{
					if (script_debug) 
					{
						log.info(log_header + 'The original CR is modifiable in the database');
					}

					// Test for not being a duplicate by itself
					// crstatus is never null
					const prefix = 'duplicate';
					var state =  originals[0].getAttribute(crstatus_attr_name).toString();
					
					if (script_debug) 
					{
						log.info(log_header + 'The original CR has the state \'' + state + '\'');
					}

					if (state.indexOf(prefix) == 0)
					{
						event.stopTransition(change.getMessage('errorDuplicatingOriginalCRXisADuplicate', [original_attr]));
					}
					else
					{
						if (script_debug) 
						{
							log.info(log_header + 'Original CR is not duplicate by itself');
						}
					}
				}
			}
		}
	}
}


/**
 * 
 * Checks the script arguments:
 * - Check argument count
 * - Check that all specified options are valid
 * - Check that all required options are specified
 * 
 * Returns: true, if succesfull. false otherwise
 */
function checkArguments(arg_count, allowed_options, required_options)
{
	var message;

	// Check argument count
	if (options.args.length < arg_count)
	{
		message = 'Insufficient arguments to run the script (' + options.args.length +' arguments)';
	}

	// Check that all specified options are valid
	if (!message)
	{
		var all_options = options.getOptions();
		var i = all_options.keySet().iterator();
	
		while (!message && i.hasNext())
		{
			var option = i.next();
			var is_valid = false;
		
			for (var j=0; j < allowed_options.length; j++)
			{
				if (option == allowed_options[j])
				{
					is_valid = true;
					break;
				}
			}

			if (!is_valid)
			{
				message = 'Invalid option \'' + option + '\'';
			}
		}
	}

	// Check that all required options are specified
	if (!message) 
	{
		for (var i=0; i < required_options.length; i++)
		{
			var option = options.getOption(required_options[i]);
			
			if (!option)
			{
				message = 'Missing option \'' + required_options[i] + '\'';
			}
		}
	}

	if (message)
	{
		log.error(log_header);
		log.error('   ' + message);
		log.error('   Usage: ' + script_usage);
		log.error('   Example: ' + script_example);
		return false;
	}

	return true;
}


/**
 * 
 * Checks, if an attribute has no value set.
 * 
 * The function is necessary, because Attribute.toString() returns 'null' instead of null. By
 * returning 'null' it cannot be distinguished between an without a value and one with the 
 * value 'null'.
 * 
 * Returns: True, if the attriute has no value set.
 */
function isAttributeNull(attribute)
{
	if (((!attribute) || (attribute.type == Attribute.Type.STRING) && (attribute.getStringValue() == null))
		|| ((attribute.type == Attribute.Type.DATE) && (attribute.getDateValue() == null))
		|| ((attribute.type == Attribute.Type.BOOLEAN) && (attribute.getBooleanValue() == null)))
	{
		return true;
	}
	
	return false;
}
