// Copyright IBM Corporation 2000, 2009
/*
<trigger>
	<usage>bsf copy_attribute.js -from attribute_name -to attribute_name [-rel relationship_name] [-if_not_state state_name]</usage>
	<version>5.2</version>
	<description>
		将当前 CR 的某个属性的值复制到同一个 CR、相关 CR 或关联任务中的指定属性。</description>
	<argument1>
		<name>-from attribute_name</name>
		<required>yes</required>
		<type>string</type>
		<description>
			从其复制值的属性的名称。</description>
	</argument1>
	<argument2>
		<name>-to attribute_name</name>
		<required>yes</required>
		<type>string</type>
		<description>
			将值复制到的属性的名称。</description>
	</argument2>
	<argument3>
		<name>-rel relationship_name</name>
		<required>no</required>
		<type>string</type>
		<description>
			指定包含目标属性的一个或多个对象。如果省略，那么将对当前 CR 设置该属性。
			如果与“associated_task”一起使用，那么将对所有关联任务设置该属性。对于任何其他值，将对与当前 CR 相关且具有关系“relationship_name”的 CR 设置该属性。</description>
	</argument3>
	<argument4>
		<name>-if_not_state state_name</name>
		<required>no</required>
		<type>string</type>
		<description>
			如果目标对象具有状态“state_name”，那么不会设置该属性。如果目标对象为 CR，那么会检查属性 crstatus 以获取状态。如果目标为任务，那么会检查属性 status。</description>
	</argument4>
</trigger>
*/


// Import packages
importPackage(Packages.com.telelogic.cs.api);

// Script specific global variables
const script_name = 'copy_attribute.js';
const script_version = '20080620';
const script_usage = 'copy_attribute.js -from attribute_name -to attribute_name [-rel relationship_name] [-if_not_state state_name]';
const script_example = 'copy_attribute.js -from resolver -to resolver -rel associated_task -if_not_state completed';
const log_header = 'Script bsf ' + script_name + ': ';
const script_debug = false;
const script_dcm_modifiable_check = true;

// Get objects
var args = bsf.lookupBean('args');
var event = bsf.lookupBean('event');
var change = bsf.lookupBean('api');
var log = bsf.lookupBean('log');
var options = OptionParser.parse(args);

if (script_debug) 
{
	log.info(log_header);
	log.info('    Version: ' + script_version);
	log.info('    Arguments: ' + args.join(' '));
	log.info('    Current CR: ' + event.objectId);
}

if (checkArguments(0, ['-from', '-to', '-rel', '-if_not_state'], ['-from', '-to']))
{
	// Get Options
	var from = options.getOption('-from');
	var to = options.getOption('-to');
	var if_not_state = options.getOption('-if_not_state');
	var rel = options.getOption('-rel');

	// Get attribute value from a change request 
	var attribute = change.getAttributes(event.triggerUser.token, event.objectId, [from]).get(from);
	
	attribute = copyAttribute(to, attribute);

	if (script_debug)
	{
		if (isAttributeNull(attribute))
		{
			log.info(log_header + 'From attribute ' + from + ' has no value');
		}
		else
		{
			log.info(log_header + 'From attribute ' + from + ' has the value \'' + attribute + '\' and is of type \'' + attribute.type + '\'');
		}
	}

	//
	// The script is from here to the end idential with set_attribute.js
	//

	// Branch over relationship type
	var state_attribute_name;
	var attribute_names = [to];
	var items;

	if (!rel)
	{
		// Self
		if (script_debug)
		{
			log.info(log_header + 'Working on self');
		}

		state_attribute_name = 'crstatus';

		if (if_not_state) 
		{
			attribute_names.push[state_attribute_name];
		}

		if (script_dcm_modifiable_check)
		{
			attribute_names.push('modifiable_in');
		}

		items = change.query(event.triggerUser.token, change.buildCrQuery(event.triggerUser.token, event.objectId), attribute_names);
	}
	else if (rel == 'associated_task')
	{
		// Associated tasks
		if (script_debug)
		{
			log.info(log_header + 'Working on associated tasks');
		}

		state_attribute_name = 'status';
		attribute_names.push('from_db_path');
		
		if (if_not_state) 
		{
			attribute_names.push(state_attribute_name);
		}

		if (script_dcm_modifiable_check)
		{
			attribute_names.push('modifiable_in');
		}

		items = change.getAssociatedTasks(event.triggerUser.token, event.objectId, attribute_names);
	}
	else
	{
		// Associated CRs
		if (script_debug) 
		{
			log.info(log_header + 'Working on associated CRs');
		}

		state_attribute_name = 'crstatus';
		
		if (if_not_state) 
		{
			attribute_names.push[state_attribute_name];
		}

		if (script_dcm_modifiable_check)
		{
			attribute_names.push('modifiable_in');
		}

		var has_string = 'has_'; 

		if (rel.indexOf(has_string) == 0)
		{
			items = change.getRelatedTo(event.triggerUser.token, event.objectId, rel.substr(has_string.length), attribute_names);
		}
		else
		{
			items = change.getRelatedFrom(event.triggerUser.token, event.objectId, rel, attribute_names);
		}
	}
	
	// Filter list
	if (script_debug) 
	{
		log.info(log_header + 'Item list: ' + items.join(', '));
	}
	items = filterItems(items, attribute, state_attribute_name, if_not_state);

	if (script_debug) 
	{
		log.info(log_header + 'Filtered item list: ' + items.join(', '));
	}
	
	// Action item
	for (var i=0; i< items.length; i++)
	{
		setObjectAttribute(items[i], attribute);
	}
}


/**
 *
 * Checks the script arguments:
 * - Check argument count
 * - Check that all specified options are valid
 * - Check that all required options are specified
 *
 * Returns: true, if succesfull. false otherwise
 */
function checkArguments(arg_count, allowed_options, required_options)
{
	var message;

	// Check argument count
	if (options.args.length < arg_count)
	{
		message = 'Insufficient arguments to run the script (' + options.args.length +' arguments)';
	}

	// Check that all specified options are valid
	if (!message)
	{
		var all_options = options.getOptions();
		var i = all_options.keySet().iterator();
	
		while (!message && i.hasNext())
		{
			var option = i.next();
			var is_valid = false;
			
			for (var j=0; j < allowed_options.length; j++)
			{
				if (option == allowed_options[j])
				{
					is_valid = true;
					break;
				}
			}
			
			if (!is_valid)
			{
				message = 'Invalid option \'' + option + '\'';
			}
		}
	}

	// Check that all required options are specified
	if (!message) 
	{
		for (var i=0; i < required_options.length; i++)
		{
			var option = options.getOption(required_options[i]);
			
			if (!option)
			{
				message = 'Missing option \'' + required_options[i] + '\'';
			}
		}
	}

	if (message)
	{
		log.error(log_header);
		log.error('   ' + message);
		log.error('   Usage: ' + script_usage);
		log.error('   Example: ' + script_example);
		return false;
	}

	return true;
}


/**
 * 
 * Checks, if an attribute has no value set.
 * 
 * The function is necessary, because Attribute.toString() returns 'null' instead of null. By
 * returning 'null' it cannot be distinguished between an without a value and one with the 
 * value 'null'.
 * 
 * Returns: True, if the attriute has no value set.
 */
function isAttributeNull(attribute)
{
	if (((!attribute) || (attribute.type == Attribute.Type.STRING) && (attribute.getStringValue() == null))
		|| ((attribute.type == Attribute.Type.DATE) && (attribute.getDateValue() == null))
		|| ((attribute.type == Attribute.Type.BOOLEAN) && (attribute.getBooleanValue() == null)))	
	{
		return true;
	}
	
	return false;
}


/**  
 * 
 * Creates a copy of a given attribute with a new name
 * 
 * The function is necessary, because new Attribute(name, null) will result in an error that
 * Attribute (java.lang.String,null) is ambiguous Attribute(java.lang.String,java.util.Date)
 * Attribute(java.lang.String,java.lang.Boolean) and Attribute(java.lang.String,java.lang.String).
 * 
 * Returns: New Attribute object
 */
function copyAttribute(name, attribute)
{
	if (isAttributeNull(attribute))
	{
		return new Attribute(name);
	}

	if (attribute.type == Attribute.Type.STRING)
	{
		return new Attribute(name, attribute.getStringValue());
	}

	if (attribute.type == Attribute.Type.DATE)
	{
		return new Attribute(name, attribute.getDateValue());
	}

	if (attribute.type == Attribute.Type.BOOLEAN)
	{
		return new Attribute(name, attribute.getBooleanValue());
	}

	log.error(log_header + 'Found unknown attribute type: ' + attribute.type);
	
	return null;
}



/** 
 * 
 * Creates an attribute object for the attribute of the specified name
 * 
 * The function is necessary to ensure that the attribute object has the correct type.
 * If created with the wrong type the attribute is cleared.
 * 
 * Returns: New Attribute object
 */
function createAttribute(name, value)
{
	if (!name)
	{
		return null;
	}

	if (!value)
	{
		return new Attribute(name);
	}

	// Lookup attribute type in the lifecycle
	var attributedefs = change.getAttributeDefs(event.triggerUser.token);
	var type;
	
	for (var i=0; i < attributedefs.length; i++)
	{
		if (attributedefs[i].name.equals(name)) // == is always false
		{
			type = attributedefs[i].type;
			break;
		}
	}

	if (!type)
	{
		log.error(log_header + 'Attribute ' + name + ' has no attribute definition in the lifecycle. Using string instead');
		return new Attribute(name, value.toString());
	}

	// The type will be the one from the lifecycle editor not Attribute.Type
	if (type == 'time')
	{
		value = new Date(value);
	}
	else if (type == 'boolean')
	{
		value = new Boolean(value);
	}
	else
	{
		value = value.toString();
	}

	return new Attribute(name, value);
}


/**
 * 
 * Filters items from the array of items to be handled by the script
 * - The to attribute has already the desired value
 * - The state is the state to be excluded
 * - The item is not modifiable in the current database and script_dcm_modifiable_check is true
 * 
 * Returns: Filtered list of items
 */
function filterItems(items, attribute, state_attribute_name, if_not_state)
{
	var new_items = new Array();
	
	for (var i = 0; i < items.length; i++)
	{
		// The to attribute has already the desired value
		// Check explicitely for empty attributes (see isAttributeNull for details) 
		if ((isAttributeNull(items[i].getAttribute(attribute.getName())) && isAttributeNull(attribute)) || 
			(!isAttributeNull(items[i].getAttribute(attribute.getName())) && !isAttributeNull(attribute) &&
			 items[i].getAttribute(attribute.getName()).toString().equals(attribute.toString())) // == is always false
			)
		{
			if (script_debug) 
			{
				log.info(log_header + 'Skipping equal value \'' + attribute.toString() + '\' in item ' + items[i]);
			}
			
			continue;
		}

		// The state is the state to be excluded
		if (items[i].getAttribute(state_attribute_name) && if_not_state && 
			items[i].getAttribute(state_attribute_name).toString().equals(if_not_state)) // == is always false
		{
			if (script_debug)
			{
				log.info(log_header + 'Skipping exluded state \'' + if_not_state + '\' in item ' + items[i]);
			}

			continue;
		}

		// The item is not modifiable in the current database and script_dcm_modifiable_check is true
		if (script_dcm_modifiable_check &&
			change.isDcmEnabled(event.triggerUser.token) &&
			!items[i].getAttribute('modifiable_in').toString().equals(change.getDbid(event.triggerUser.token)) &&
			!(change.isCentralServer(event.triggerUser.token) && items[i] instanceof Task)) // == is always false
		{
			if (script_debug)
			{
				log.info(log_header + 'Skipping non modifiable item. Item \'' + items [i]+ 
				'\' is not modifiable in database \'' + change.getDbid(event.triggerUser.token) + '\'');
			}

			continue;
		}

		new_items.push(items[i]);
	}

	return new_items;
}


/** 
 * 
 * Sets an attribute on a CR or task
 */
function setObjectAttribute(item, attribute)
{
	var is_task = (item.toString().indexOf('Task') == 0);
	var attributes = [attribute];
	var item_id_string;

	try 
	{
		if (is_task)
		{
			// Set attribute on a task
			item_id_string = 'Task ' +  item.taskId;

			if (script_debug) 
			{
				log.info(log_header + 'Modifying ' + item_id_string);
			}
	
			// Override the context database in central server mode
			var from_db_path = item.getAttribute('from_db_path');
			
			if (from_db_path)
			{
				attributes.push[from_db_path];
			}
	
			change.setTaskAttributes(event.triggerUser.token, item.taskId, attributes);	
		} 
		else 
		{
			// Set attribute on a CR
			item_id_string = 'CR ' +  item.crid;

			if (script_debug)
			{
				log.info(log_header + 'Modifying ' + item_id_string);
			}
	
			change.setAttributes(event.triggerUser.token, item.crid, attributes);	
		}
	}
	catch (error)
	{
		var message = change.getMessage('errorSettingAttributeXonItemYerrorZ', [attribute.name, item_id_string, error.message || error.toString()]);	
		log.error(log_header + 'Copying from CR ' + event.objectId + '. ' + message); 
		event.fail(message);
	}
}
